/*
*  Rubrica
*  file: file.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>
#include <gnome.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

#include "rubrica.h"
#include "modulo.h"
#include "main.h"
#include "file.h"

VistaDati vista;

static gchar *file;
static gchar *path;
static gchar *nomefile;

void rubrica_init_archivio(void)
{
  gint n;    
  gchar *str;
  InfoRubrica i;
  GtkCList *schede;
  GtkCList *dati;
  
  /* libero la memoria occupata da rubrica */
  /* rubrica = NULL, impostato da rubrica_svuota_memoria richiamando 
     rubrica_set_rubrica
  */
  rubrica_svuota_memoria();

  /* setto a FAlSE il flag che controlla la presenza della rubrica */
  rubrica_set_empty(TRUE);
  
  /* azzero il numero totale di indirizzi */
  n = 0;
  rubrica_vista_set_indirizzi(n);
  
  /* cancello il nome del file visualizzato */
  str = g_strdup("");
  rubrica_vista_set_label(str);
  g_free(str);

  /* imposto i valori booleani sullo stato della rubrica */
  rubrica_modificata(FALSE);

  /* inizializzo i valori della clist contenente 
     l'elenco delle schede 
  */
  i.riga = 0;
  i.colonna = 0;
  i.selezionato = FALSE;
  rubrica_vista_set_info(i, VISTA_SCHEDE);
  rubrica_vista_set_info(i, VISTA_DATI);
  
  /* pulisco le clist
     - schede
     - dati
  */
  schede = rubrica_vista_get_schede();
  dati   = rubrica_vista_get_dati();
  
  gtk_clist_clear(GTK_CLIST(schede));
  gtk_clist_clear(GTK_CLIST(dati));  
  
  /* inizializzo i ptr ai nomi del file */
  if (file)
    g_free(file);
  if (path)
    g_free(path);
  if(nomefile)
    g_free(nomefile);
  
  file = NULL;
  path = NULL;
  nomefile = NULL;
}

void 
rubrica_crea_nuovo_archivio(void)
{
  gint scelta;
  gchar *mes;
  
  if (rubrica_is_modificata())
    {
      /*
      mes = g_strdup(_("Attenzione. La rubrica  stata modificata.\nI cambiamenti non salvati saranno persi.\n Continuo?")); 
      */
      mes = g_strdup(_("Warning. Address book was changed.\nChanges not saved, will be lose.\nContinue?")); 
      
      scelta = confermare_uscita(mes);
      g_free(mes);
      
      switch(scelta)
	{
	case 0:
	  rubrica_init_archivio();
	  break;
	  
	case 1:
	default:
	  break;
	}  
    }
  else 
    rubrica_init_archivio();   
}



void 
rubrica_salva_file(GtkWidget *w, gpointer data)
{
  GtkWidget *app;

  app = rubrica_get_app();
  /* se non c' un nome file di default
     allora bisogna richiamare salva_con_nome
  */
  if (!nomefile)
    rubrica_salva_file_come(NULL, NULL);
  /* altrimenti provo a scrivere il file */
  else 
    if (!write_xml_file(nomefile)) 
/*	gnome_app_error(GNOME_APP(app), _("Impossibile salvare il file")); */
	gnome_app_error(GNOME_APP(app), _("I can't save the file"));
}

void 
rubrica_salva_file_come(GtkWidget *w, gpointer data)
{
  GtkWidget *app;
  GtkFileSelection *fs;
  
  app = rubrica_get_app();
/*
  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Salva come")));
*/
  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Save as")));
  
  /* se ho gi un nome di file lo mostro nell'entry*/
  if (nomefile)
       gtk_file_selection_set_filename(fs, nomefile);
  
  /* connetto i bottoni ok e cancella */
  gtk_signal_connect(GTK_OBJECT(fs->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(rubrica_scrivi_file), fs);
  
  /* con il bottone cancella distruggo direttamente il widget */
  gtk_signal_connect_object(GTK_OBJECT(fs->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(fs));

  gtk_window_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));
  
  gtk_widget_show(GTK_WIDGET(fs));
}


void
rubrica_scrivi_file(GtkWidget *w, GtkFileSelection *fs)
{
  gchar *fname;

  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));

  fname = gtk_file_selection_get_filename(fs);

  /* se non  stato selezionato nessun file
     o non  possibile aprire il file segnalare l'errore */
  if ( !fname || !write_xml_file(fname)) {
    GtkWidget *dlg;

    /* il dialogo di errore ha il widget fs come genitore */
/*
    dlg = gnome_error_dialog_parented(_("Impossibile salvare il file"), 
				      GTK_WINDOW(fs));
*/
    dlg = gnome_error_dialog_parented(_("I can't save the file"), 
				      GTK_WINDOW(fs));
    gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);   
  }
  else /* il file  stato salvato */
    {
      /* imposto l'etichetta del nome file */
      if (rubrica_get_mostra_path())
	rubrica_vista_set_label(path);
      else
	rubrica_vista_set_label(nomefile);
      
      gtk_widget_destroy(GTK_WIDGET(fs));
    }
}

gboolean
write_xml_file(gchar *fname)
{
  /* puntatori al documento xml */
  xmlDocPtr doc;
  GList *l;
  GList *rub;
    
  rub = rubrica_get_rubrica();

  g_return_val_if_fail(fname != NULL, FALSE);
  g_return_val_if_fail(rub != NULL, FALSE);
  
  /* creazione di un nuovo documento xml */
  doc = xmlNewDoc("1.0");
  
  /* creo un nuovo nodo radice rubrica */
  doc->root = xmlNewDocNode(doc, NULL, "Rubrica", NULL);

  /* attraverso la rubrica e scrivo i dati */
  for (l = rub; l != NULL; l = l->next) {
    RubricaItem *item = l->data;

    rubrica_write_elemento_to_xml(doc->root, item);
  }

  if (xmlSaveFile(fname, doc) == -1) {
    xmlFreeDoc(doc);
    return FALSE;
  }
  if (file)
    g_free(file);
  if (path)
    g_free(path);
  if(nomefile)
    g_free(nomefile);
  
  file = g_strdup(fname);
  path = g_strdup(g_dirname(fname));
  nomefile = g_strdup(g_basename(fname));

  xmlFreeDoc(doc);    

  return TRUE;
}

void 
rubrica_write_elemento_to_xml(xmlNodePtr xmlnode, RubricaItem *item)
{
  GList *alias;

  xmlNodePtr newxml;

  xmlNodePtr schedaxml;
  xmlNodePtr datixml;
  xmlNodePtr webxml;
  xmlNodePtr mailxml;
  xmlNodePtr telxml;
  xmlNodePtr notexml;

  xmlNodePtr nodo;

  g_return_if_fail(xmlnode != NULL);
  g_return_if_fail(item != NULL);

  /*
    newxml  figlio di xmlnode (il primo xmlnode  doc->root)
    boolxml, schedaxml, datixml, ..., sono figli di newxml
  */
    
  newxml = xmlNewChild(xmlnode, NULL, "Indirizzo", NULL);
  
  /*
    xmlNewTextChild 
    sintassi :
         - nodo genitore del nodo creato con la chiamata
	 - namespace
	 - nome del nodo creato
	 - testo (elemento) da inserire nel nodo creato
  */
  schedaxml = xmlNewTextChild(newxml, NULL, "Scheda", item->scheda);

  /* setto le propriet della scheda (editabile, concellabile) */
  if (item->editabile)
    xmlSetProp(schedaxml, "editabile", "TRUE");
  else
    xmlSetProp(schedaxml, "editabile", "FALSE");
  if (item->cancellabile)
    xmlSetProp(schedaxml, "cancellabile", "TRUE");
  else
    xmlSetProp(schedaxml, "cancellabile", "FALSE");
  
  /* ho creato l'elemento xml es.: 
     <Scheda editabile="TRUE" cancellabile="FALSE>scheda numero uno</Scheda>
  */
  
  datixml = xmlNewTextChild(newxml, NULL, "Dati", NULL);
  nodo = xmlNewTextChild(datixml, NULL, "Nome", item->nome);
  nodo = xmlNewTextChild(datixml, NULL, "Cognome", item->cognome);
  nodo = xmlNewTextChild(datixml, NULL, "Professione", item->professione);
  nodo = xmlNewTextChild(datixml, NULL, "Via", item->via);
  nodo = xmlNewTextChild(datixml, NULL, "Numero", item->numero);
  nodo = xmlNewTextChild(datixml, NULL, "Cap", item->cap);
  nodo = xmlNewTextChild(datixml, NULL, "Citt", item->citta);
  nodo = xmlNewTextChild(datixml, NULL, "Provincia", item->provincia);
  nodo = xmlNewTextChild(datixml, NULL, "Stato", item->stato);
 
  webxml = xmlNewChild(newxml, NULL, "Indirizzi_Web", NULL);

  alias = item->web_list;
  while(alias)
    {
      nodo = xmlNewTextChild(webxml, NULL, "Web", alias->data);
      
      alias = alias->next;
    }
  
  mailxml = xmlNewChild(newxml, NULL, "Indirizzi_Email", NULL);
  alias = item->mail_list;
  while(alias)
    {
      nodo = xmlNewTextChild(mailxml, NULL, "Mail", alias->data);

      alias = alias->next;
    }

  telxml = xmlNewChild(newxml, NULL, "Numeri_Telefonici", NULL);
  alias = item->tel_list;
  while(alias)
    {
      TelNum *tt = (TelNum*) alias->data;

      nodo = xmlNewTextChild(telxml, NULL, "Telefono", tt->numero_telefono);

      xmlSetProp(nodo, "Tipo", tt->tipo);
      
      alias = alias->next;
    }
  
  notexml = xmlNewTextChild(newxml, NULL, "Note", item->note);
}


void 
rubrica_carica_file(void)
{
  GtkFileSelection *fs;
  GtkWidget *app;

  app = rubrica_get_app();

/*
  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Apri file...")));
*/
  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Open file...")));
  
  /* connetto i bottoni ok e cancel */
  gtk_signal_connect(GTK_OBJECT(fs->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(rubrica_apri_file), fs);
  gtk_signal_connect_object(GTK_OBJECT(fs->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(fs));

  /* visualizzo la finestra in posizione centrale */
  gtk_window_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);

  /* il genitore della finestra fileselection  app */
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));

  gtk_widget_show(GTK_WIDGET(fs));
}

void 
rubrica_apri_file(GtkWidget *w, GtkFileSelection *fs)
{
  gchar *fname;

  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));

  fname = gtk_file_selection_get_filename(fs);

  if (!fname || !read_xml_file(fname)) 
    {
      /* File not opened 
	 error:
	 - bad filename 
	 - error reading file
      */
      GtkWidget *dlg;
/*      
      dlg = gnome_error_dialog_parented(_("Impossibile aprire il file"), 
					GTK_WINDOW(fs));
*/
      dlg = gnome_error_dialog_parented(_("I can't open the file"), 
					GTK_WINDOW(fs));
      gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);
    }
  else /* Ok file read */
    {
      if (rubrica_get_mostra_path())
	rubrica_vista_set_label(path);
      else
	rubrica_vista_set_label(nomefile);

      gtk_widget_destroy(GTK_WIDGET(fs));
      rubrica_set_empty(FALSE);
    }
}


gboolean 
read_xml_file(gchar *fname)
{
  xmlDocPtr doc;

  g_return_val_if_fail(fname != NULL, FALSE);
  
  /* se il file non esiste */
  if (!g_file_exists(fname))
    return FALSE;

  doc = xmlParseFile(fname);

  if (!doc)
    return FALSE;

  if (!doc->root || !doc->root->name || 
      g_strcasecmp(doc->root->name, "Rubrica") != 0)
    {
      xmlFreeDoc(doc);
      return FALSE;
    }
  parse_doc(doc);

  xmlFreeDoc(doc);

  if (file)
    g_free(file);
  if(path)
    g_free(path);
  if(nomefile)
    g_free(nomefile);
  
  file = g_strdup(fname);
  path = g_strdup(g_dirname(fname));
  nomefile = g_strdup(g_basename(fname));
  
  return TRUE;  
}


void 
parse_doc(xmlDocPtr doc)
{  
  xmlNodePtr nodo;

  /* inizializzo le strutture necessarie per 
     caricare una nuova lista
  */
  rubrica_crea_nuovo_archivio();

  /* prendo il nodo Indirizzo dal file e lo inserisco nella 
     (nuova) rubrica 
     il primo nodo  doc->root->childs == xmlnew (salvato)
  */
  for(nodo = doc->root->childs; nodo != NULL; nodo=nodo->next)
    rubrica_load_elemento_from_xml(nodo, NULL);  
}

void
rubrica_load_elemento_from_xml(xmlNodePtr xmlnode, GList *l)
{
  gchar *buffer;
  RubricaItem *r;  
  GList *web, *mail, *tel;
  xmlNodePtr figlio;

  /* i nodi xmlnode sono nodi xmlnew
     - devo estrarre i figli di xmlnew == xmldoc che sono:
       schedaxml, ...
  */
  
  if (!xmlnode->name ||  /* se il nodo non ha nome o */
                         /* il nome non  Indirizzo  */
      g_strcasecmp(xmlnode->name, "Indirizzo") != 0) 
    return;
    
  r = g_malloc(sizeof(RubricaItem));

  /* schedaxml */
  figlio = xmlnode->childs;
  if (!figlio->name || g_strcasecmp(figlio->name, "Scheda") != 0)   
    {
      r->scheda = g_strdup("");
    }
  else     
    r->scheda = g_strdup(xmlNodeGetContent(figlio));
    
  /* scheda ha degli attributi, bisogna estrarli */
  buffer = xmlGetProp(figlio, "editabile");
  if (g_strcasecmp(buffer, "TRUE") == 0)
    r->editabile = TRUE;
  else 
    r->editabile = FALSE;
  
  buffer = xmlGetProp(figlio, "cancellabile");
  if (g_strcasecmp(buffer, "TRUE") == 0)
    r->cancellabile = TRUE;
  else 
    r->cancellabile = FALSE;  
  
  /* dati personali -- datixml -- */
  figlio = figlio->next; 
  if (!figlio->name || g_strcasecmp(figlio->name, "Dati") != 0)
    r->nome = r->cognome 
      = r->professione = r->via 
      = r->numero = r->cap 
      = r->citta = r->provincia 
      = r->stato = g_strdup("");
  else 
    {
      /* carico i figli di datixml
	   - datixml=figlio 
	   - nodo = figlio di datixml
      */
      xmlNodePtr nodo;

      nodo = figlio->childs;
      r->nome = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->cognome = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->professione = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->via = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->numero = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->cap = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->citta = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->provincia = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->stato = g_strdup(xmlNodeGetContent(nodo));
    }

  /* web */
  web = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Indirizzi_Web") != 0)
    {
      r->web_list = NULL;
      r->nweb = 0;
    }
  else
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  buffer = g_strdup(xmlNodeGetContent(nodo));
	  
	  web = g_list_append(web, buffer);
	  nodo = nodo->next;
	  n++;
	}
      r->web_list = web;
      r->nweb = n;
    }
  
  /* mail */
  figlio = figlio->next;
  mail = NULL;
  if (!figlio->name || g_strcasecmp(figlio->name, "Indirizzi_Email") != 0)
    {
      r->mail_list = NULL;
      r->nmail = 0;
    }
  else
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;

      nodo = figlio->childs;
      while(nodo)
	{
	  buffer = g_strdup(xmlNodeGetContent(nodo));

	  mail = g_list_append(mail, buffer);
	  nodo = nodo->next;
	  n++;
	}
      r->mail_list = mail;
      r->nmail = n;
    }  
    
  /* telefono */
  tel = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Numeri_Telefonici") != 0)
    {
      r->tel_list = NULL;
      r->ntel = 0;
    }
  else
    {
      gint n=0;
      xmlNodePtr nodo;
      TelNum *t;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  t = g_malloc(sizeof(TelNum));
	  
	  t->numero_telefono = g_strdup(xmlNodeGetContent(nodo));
	  t->tipo = g_strdup(xmlGetProp(nodo,"Tipo"));
	  
	  tel = g_list_append(tel, t);
	  nodo = nodo->next;
	  n++;
	}
      r->tel_list = tel;
      r->ntel = n;
    }
  
  /* note */
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Note") != 0)
    r->note = g_strdup("");
  else 
    r->note = g_strdup(xmlNodeGetContent(figlio));
  
  rubrica_inserisci_elemento(r);
  rubrica_vista_inc_indirizzi();
  rubrica_inserisci_scheda(r);
}





