/*
*  Rubrica
*  file: import.h
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include "rubrica.h"
#include "file.h"
#include "error.h"
#include "import.h"
#include "import_int.h"

ImportFrom importfrom;

RubricaItem *r;
TelNum *tt;
GList *web=NULL, *mail=NULL, *tel=NULL;
GList *note;

TokenCard token;
SubToken subtoken=SUB_BAD;

static gchar *gcard_default="/.gnome/GnomeCard.gcrd";
static gchar *card_kde1="/.kde/share/apps/kab/addressbook.database";
static gchar *card_kde2="";
gint ntel, nmail, n;
gint notelen;

void import_from_init(void)
{
  importfrom = CVS; /* import default: cvs file */
}

void import_from_set(ImportFrom i)
{
  importfrom = (ImportFrom) i;
}

ImportFrom import_from_get(void)
{
  return (ImportFrom) importfrom;
}


void import_data(void)
{
  GtkWidget *fentry;
  gchar *filename;
  gchar *buffer, *titolo;
  
  n=0;
  fentry = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY
				      (rubrica_import_get_file_entry()));

  filename = gtk_entry_get_text(GTK_ENTRY(fentry));

  /* creao una nuova lista rubrica, e 
     salvo una eventuale lista precedente */
  rubrica_crea_nuovo_archivio();

  switch (importfrom)
    {
    case GCARD:
      if (strcmp(filename,"") == 0)
	{
	  /* carico il file di default */
	  buffer = g_get_home_dir();
	  
	  filename = g_strdup_printf("%s%s",buffer,gcard_default);
	  rubrica_import_open_file(filename);
	  titolo = _("Imported from GnomeCard");
	}          
      break;
      
    case KCARD:
      /* kab (kde 1.x) ha un database di default */
      buffer =  g_get_home_dir();
      
      filename = g_strdup_printf("%s%s",buffer,card_kde1);
      rubrica_import_open_file(filename);
      titolo = _("Imported from Kab");
      break;

    case CVS:
    default:
      if (strcmp(filename,"") == 0)
	{
	  g_print("\nerrore. selezionare un file ");
	  return;
	}
      else 
	{
	rubrica_import_open_file(filename);      
	titolo = _("Imported from Netscape cvs");
	
	}
      break;      
    }
  rubrica_vista_set_indirizzi(n);
  rubrica_modificata(TRUE);
  rubrica_vista_set_label(titolo);
/*  rubrica_set_titolo(titolo); */
  rubrica_set_empty(FALSE);
}


void rubrica_import_open_file(gchar *file)
{
  FILE *fp;
//  gboolean esiste;

  if ((fp=fopen(file, "r")) == NULL)
    {
      /* messaggio di errore */ 
      g_print("\nerrore, impossibile leggere il file");
      errore_import_no_file();

      return;
    }

  while (!feof(fp))
    {
      gchar buffer[255];

      while (fgets(buffer, 255, fp))
	{	  
	  g_print("decodificare -- %s\n", buffer);
	  rubrica_decode_buffer(buffer);
	}
    }
}

void export_data(void)
{
}

void rubrica_decode_buffer(gchar *buffer)
{
  gchar *str=NULL;
  gchar *buf=NULL, *tmp=NULL, *notebuf, *orgbuf, *revbuf;
  gchar *decoded[8];
 
  str = g_strdup(buffer);
  str = strtok(str, ":");  
  token = decode_token(str);

  switch(token)
    {
    case BEGIN:
      /* allocare lo spazio per un nuovo indirizzo */
      r = (RubricaItem*) g_malloc(sizeof(RubricaItem));
      ntel = nmail = notelen = 0;
      import_init_item(r);
      break;
      
      /*
	case VCARD:
	non fare nulla,  compreso in begin 
	break;
      */

    case FN:
      /* nome della scheda */ 
      str = strtok(NULL, ":");
      r->scheda = g_strdup(str);      
      break;
      
    case N:
      /* ordine:: cognome, nome, secondo nome, prefisso, suffisso */ 
      buf = import_get_string(buffer);
      import_decode_string(buf, decoded);
      g_free(buf);
 
      r->cognome = g_strdup(decoded[0]);
      r->nome = g_strdup(decoded[1]);
      /*
	decoded[2] <= second name
	decoded[3] <= prefix
	decoded[4] <= suffix
       */
      tmp = g_strdup_printf(_("Second name: %s"),decoded[2]);
      notelen += strlen(tmp);
      note = g_list_append(note, tmp);
      tmp = g_strdup_printf(_("Prefix: %s"),decoded[3]);
      notelen += strlen(tmp);
      note = g_list_append(note, tmp);     
      tmp = g_strdup_printf(_("Suffix: %s"),decoded[4]);
      notelen += strlen(tmp);
      note = g_list_append(note, tmp);      
      break;

    case REV:
      /* data, fuso orario, ora */
      str = strtok(NULL, ";");
      revbuf = g_strdup_printf(_("Fuso orario: %s"),str);
      notelen += strlen(revbuf);
      note = g_list_append(note, revbuf);
      break;

    case BDAY:
      str = strtok(NULL, ":");
      buf = g_strdup_printf(_("Birthday: %s"),str);
      notelen += strlen(buf);
      note = g_list_append(note, buf);
      break;

    case ADR:
      /* indirizzo di posta + tipo */
      buf = import_get_string(buffer);
      import_decode_string(buf, decoded);
      g_free(buf);

      r->via = g_strdup(decoded[2]);
      r->numero = g_strdup("");
      r->cap = g_strdup(decoded[5]);
      r->citta = g_strdup(decoded[3]);
      r->provincia = g_strdup(decoded[4]);
      r->stato = g_strdup(decoded[6]);
      /* 
	 decoded[0] <= post office
	 decoded[1] <= extended
	 decoded[4] <= region ==> province
      */
      tmp = g_strdup_printf(_("Post Office: %s"),decoded[0]);
      notelen += strlen(tmp);
      note = g_list_append(note, tmp);      
      tmp = g_strdup_printf(_("Extended: %s"),decoded[1]);
      notelen += strlen(tmp);
      note = g_list_append(note, tmp);      
      break;

    case TEL:
      /* telefono + tipo */
      tt = (TelNum*) g_malloc(sizeof(TelNum));

      switch (subtoken)
	{
	case HOME:
	  str = import_get_string(buffer);
	  tt->numero_telefono = str;
	  tt->tipo = "Casa";
	  tel = g_list_append(tel, tt);
	  ntel++;
	  break;
	case WORK:
	  str = import_get_string(buffer);
	  tt->numero_telefono = str;
	  tt->tipo = "Ufficio";
	  tel = g_list_append(tel, tt);
	  ntel++;
	  break;
	case FAX:
	  str = import_get_string(buffer);
	  tt->numero_telefono = str;
	  tt->tipo = "Fax";
	  tel = g_list_append(tel, tt);	  
	  ntel++;
	  break;
	case CELL:
	  str = import_get_string(buffer);
	  tt->numero_telefono = str;
	  tt->tipo = "Cellulare";
	  tel = g_list_append(tel, tt);
	  ntel++;
	  break;
	  
	  /* altro */
	case PREF:
	case VOICE:
	case MSG:
	case PAGER:
	case BBS:
	case MODEM:
	case CAR:
	case ISDN:
	case VIDEO:
	default:
	  str = import_get_string(buffer);
	  tt->numero_telefono =str;
	  tt->tipo = "Altro";
	  tel = g_list_append(tel, tt);	  
	  ntel++;
	  break;
	} 
      break;
      
    case EMAIL:      
      /* email +tipo */
      switch(subtoken)
	{
	case AOL:
	case INTERNET:
	case APPLELINK:
	case ATTMAIL:
	case CIS:
	case EWORLD:
	case IBMMail:
	case MCIMAIL:
	case POWERSHARE:
	case PRODIGY:
	case TLX:
	case X400:
	default:
	  buf = import_get_string(buffer);
	  mail = g_list_append(mail, buf);	  
	  nmail++;
	  break;
	}
      break;

    case URL:
      buf = import_get_string(buffer); 
      buf = g_strdup_printf("http:%s",buf);
      web = g_list_append(web, buf);      
      r->web_list = web;
      r->nweb = 1;
      /* url */
      break;
      
    case TITLE:
      /* titolo nell'organizzazione (professione) */
      str = strtok(NULL, ":");
      r->professione = g_strdup(str);
      break;

    case ORG:
      /* nome dell'organizzazione */
      str = strtok(NULL, ":");
      orgbuf = g_strdup_printf(_("Org. Name: %s"),str);
      notelen += strlen(orgbuf);
      note = g_list_append(note, orgbuf);
      break;

    case CATEGORIES:
      break;

    case NOTE:
      str = strtok(NULL, ":");
      notebuf = g_strdup_printf(_("Other notes: %s"),str);     
      notelen +=strlen(notebuf);
      note = g_list_append(note, notebuf);
      
      /* note */
      break;

    case END:
      /* collegare le liste mail e tel */
      r->mail_list = mail;
      r->tel_list = tel;
      r->nmail = nmail;
      r->ntel = ntel;
      r->note = g_strdup(import_decode_note(note));
      import_free_note(note);
      /* inserire l'elemento nella lista rubrica */
      rubrica_inserisci_elemento(r);
      n++;
      /* impostare il flag di lista vuota a falso */
      /* visualizzo la nuova scheda in lista */
      rubrica_inserisci_scheda(r);      
      break;

    case VCARD:  
    case BAD:
    default:
      break;
    }
}

TokenCard decode_token(gchar *str)
{
  gchar *tmp;

  if ((strcmp(str, "BEGIN"))==0)
    {
      str = strtok(NULL, "\0");
      if ((strcmp(str,"VCARD")== 0))
	g_print("\nvcard");
      else
	g_print("\nerror decoding BEGIN:VCARD");
      
      return (TokenCard) BEGIN;
    }

  if ((strcmp(str,"END")) == 0)
    {
      str = strtok(NULL, "\0");
      if ((strcmp(str,"VCARD")== 0))
	g_print("\nvcard");
      else
	g_print("\nerror decoding END:VCARD");
      
      return (TokenCard) END;
    }

  if ((strcmp(str, "FN"))==0)
    return (TokenCard) FN;

  if ((strcmp(str, "N"))==0)
    return (TokenCard) N;

  if ((strcmp(str, "REV"))==0)
    return (TokenCard) REV;

  if ((strcmp(str, "BDAY"))==0)
    return (TokenCard) BDAY;  

  if ((strcmp(str,"TITLE")) == 0)
    return (TokenCard) TITLE;

  if ((strcmp(str,"ORG")) == 0)
    return (TokenCard) ORG;

  if ((strcmp(str,"CATEGORIES")) == 0)
    return (TokenCard) CATEGORIES;

  if ((strcmp(str,"NOTE")) == 0)
    return (TokenCard) NOTE;

  if ((strcmp(str,"URL")) == 0)
    return (TokenCard) URL;

  /* token composti */
  
  tmp = g_strdup(str);
  tmp = strtok(tmp,";");
  if ((strcmp(tmp,"ADR")) == 0)
    {
      tmp = strtok(NULL,";");
      subtoken = decode_subtoken(tmp);

      return (TokenCard) ADR;
    }

  if ((strcmp(tmp,"TEL")) == 0)
    {
      tmp = strtok(NULL,";");
      subtoken = decode_subtoken(tmp);  
      
      return (TokenCard) TEL;
    }

  if ((strcmp(tmp,"EMAIL")) == 0)
    {
      tmp = strtok(NULL,";");
      subtoken = decode_subtoken(tmp);
      
      return (TokenCard) EMAIL;
    }

  return BAD; 
}

SubToken decode_subtoken(gchar *str)
{
  if ((strcmp(str,"HOME")) == 0)
    return (SubToken) HOME;

  if ((strcmp(str,"WORK")) == 0)
    return (SubToken) WORK;

  if ((strcmp(str,"POSTAL")) == 0)
    return (SubToken) POSTAL;

  if ((strcmp(str,"PARCEL")) == 0)
    return (SubToken) PARCEL;

  if ((strcmp(str,"DOM")) == 0)
    return (SubToken) DOM;

  if ((strcmp(str,"INTL")) == 0)
    return (SubToken) INTL;

  if ((strcmp(str,"PREF")) == 0)
    return (SubToken) PREF;

  if ((strcmp(str,"VOICE")) == 0)
    return (SubToken) VOICE;

  if ((strcmp(str,"FAX")) == 0)
    return (SubToken) FAX;

  if ((strcmp(str,"MSG")) == 0)
    return (SubToken) MSG;

  if ((strcmp(str,"CELL")) == 0)
    return (SubToken) CELL;

  if ((strcmp(str,"PAGER")) == 0)
    return (SubToken) PAGER;

  if ((strcmp(str,"BBS")) == 0)
    return (SubToken) BBS;

  if ((strcmp(str,"MODEM")) == 0)
    return (SubToken) MODEM;

  if ((strcmp(str,"CAR")) == 0)
    return (SubToken) CAR;

  if ((strcmp(str,"ISDN")) == 0)
    return (SubToken) ISDN;

  if ((strcmp(str,"VIDEO")) == 0)
    return (SubToken) VIDEO;

  if ((strcmp(str,"AOL")) == 0)
    return (SubToken) AOL;

  if ((strcmp(str,"INTERNET")) == 0)
    return (SubToken) INTERNET;

  if ((strcmp(str,"APPLELINK")) == 0)
    return (SubToken) APPLELINK;

  if ((strcmp(str,"ATTMAIL")) == 0)
    return (SubToken) ATTMAIL;

  if ((strcmp(str,"CIS")) == 0)
    return (SubToken) CIS;

  if ((strcmp(str,"EWORLD")) == 0)
    return (SubToken)EWORLD ;

  if ((strcmp(str,"IBMMail")) == 0)
    return (SubToken)IBMMail ;

  if ((strcmp(str,"MCIMAIL")) == 0)
    return (SubToken) MCIMAIL;

  if ((strcmp(str,"POWERSHARE")) == 0)
    return (SubToken) POWERSHARE;

  if ((strcmp(str,"PRODIGY")) == 0)
    return (SubToken) PRODIGY;

  if ((strcmp(str,"TLX")) == 0)
    return (SubToken) TLX;

  if ((strcmp(str,"X400")) == 0)
    return (SubToken) X400;

  return SUB_BAD;
}

gchar *import_get_string(gchar *buf)
{     
  gchar *str;
  
  g_strreverse(buf);
  str = strtok(buf, ":");
  g_strreverse(str);
  
  return (gchar*) g_strdup(str);	      
}

void import_decode_string(char *buf, gchar *dec[7])
{
  gint i, j, k;
  gchar *tmp;

  /* inizializzazione buffer decodificato */
  for(i=0; i<8;i++)
    dec[i]=NULL;

  tmp = g_malloc (strlen(buf));
  i = 0;
  j = 0;
  k = 0;
  while (buf[j] != '\0')
    {
      if(buf[j] != ';')
	{
	  tmp[k] = buf[j];
	  j++;
	  k++;
	}	 
      else
	{
	  tmp[k]='\0';
	  dec[i] = g_strdup(tmp);
	  i++;
	  j++;
	  k=0;
	}		
    }
  tmp[k]='\0';
  dec[i] = g_strdup(tmp); 
}


void import_init_item(RubricaItem *r)
{
  r->editabile = r->cancellabile = TRUE;      

  r->scheda = r->nome = r->cognome = r->professione = r->via = NULL;
  r->numero = r->cap = r->citta = r->provincia = r->stato = r->note = NULL;

  r->web_list = web;
  r->mail_list = mail;
  r->tel_list = tel;      

  r->nweb = r->nmail = r->ntel = 0;
  note = NULL;  
}

gchar *import_decode_note(GList *l)
{
  gchar *nt, *tmp;


  nt = (gchar *) g_malloc(sizeof(gchar)*(notelen+1));
  *nt = '\0';
  while(note)
    {
      tmp = (gchar*) note->data;     
      strncat(nt, tmp, strlen(tmp));
      note = g_list_next(note);
    }   

  return (gchar *) nt;
}

void import_free_note(GList *n)
{
  gchar *tmp;
  GList *nodo;

  while (n)
    {
      nodo = g_list_last(note);
      n = g_list_remove_link(n, nodo);
      
      tmp = nodo->data;
      if (tmp)
	free(tmp);
      
      g_list_free_1(nodo);      
    } 
  n = NULL;
}
